@@ -1,4 +1,4 @@
-/* $Id: pmk.c,v 1.128 2004/02/12 12:25:30 mipsator Exp $ */
+/* $Id: pmk.c,v 1.136 2004/12/26 23:42:11 mipsator Exp $ */
 
 /*
  * Credits for patches :
@@ -40,7 +40,7 @@
 
 
 #include <sys/stat.h>
-/* include it first as if it was <sys/types.h> - this will avoid errors */ 
+/* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
 
 #include <errno.h>
@@ -56,7 +56,6 @@
 #include "func.h"
 #include "pathtools.h"
 #include "pmk.h"
-#include "parse.h"
 
 /*#define PMK_DEBUG	1*/
 
@@ -95,14 +94,24 @@ bool process_dyn_var(pmkdata *pgd, char *template) {
 	/* get template path */
 	/* NOTE : we use strdup to avoid problem with linux's dirname */
 	pstr = strdup(template);
-	strlcpy(stpath, dirname(pstr), sizeof(stpath));
+	if (pstr == NULL) {
+		free(pstr);
+		errorf(ERRMSG_MEM);
+		return(false);
+	}
+
+	if (strlcpy_b(stpath, dirname(pstr), sizeof(stpath)) == false) {
+		free(pstr);
+		errorf(PMK_ERR_OVRFLOW);
+		return(false);
+	}
 	free(pstr);
 
 	/* compute relative path */
 	relpath(srcdir, stpath, rpath);
 
 	/* compute builddir_abs with relative path */
-	abspath(basedir, rpath, btpath); 
+	abspath(basedir, rpath, btpath);
 	if (hash_update_dup(pht, PMK_DIR_BLD_ABS, btpath) == HASH_ADD_FAIL)
 		return(false);
 #ifdef PMK_DEBUG
@@ -208,18 +217,18 @@ bool init_var(pmkdata *pgd) {
 	pht = pgd->htab;
 
 	pstr = MK_VAR_CFLAGS;
-	if (get_make_var(pstr, buf, sizeof(buf)) == true) {
-		if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-		return(false);
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
 	}
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
 #endif
+	if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
+		return(false);
 
 	pstr = MK_VAR_CXXFLAGS;
-	if (get_make_var(pstr, buf, sizeof(buf)) == true) {
-		if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-		return(false);
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
 	}
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
@@ -228,9 +237,8 @@ debugf("%s = '%s'", pstr, buf);
 		return(false);
 
 	pstr = MK_VAR_CPPFLAGS;
-	if (get_make_var(pstr, buf, sizeof(buf)) == true) {
-		if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-			return(false);
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
 	}
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
@@ -239,9 +247,8 @@ debugf("%s = '%s'", pstr, buf);
 		return(false);
 
 	pstr = MK_VAR_LDFLAGS;
-	if (get_make_var(pstr, buf, sizeof(buf)) == true) {
-		if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-			return(false);
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
 	}
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
@@ -250,9 +257,8 @@ debugf("%s = '%s'", pstr, buf);
 		return(false);
 
 	pstr = MK_VAR_LIBS;
-	if (get_make_var(pstr, buf, sizeof(buf)) == true) {
-		if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-			return(false);
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
 	}
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
@@ -261,9 +267,8 @@ debugf("%s = '%s'", pstr, buf);
 		return(false);
 
 	pstr = MK_VAR_DEBUG;
-	if (get_make_var(pstr, buf, sizeof(buf)) == true) {
-		if (hash_update_dup(pht, pstr, buf) == HASH_ADD_FAIL)
-			return(false);
+	if (get_make_var(pstr, buf, sizeof(buf)) == false) {
+		buf[0] = CHAR_EOS; /* empty string */
 	}
 #ifdef PMK_DEBUG
 debugf("%s = '%s'", pstr, buf);
@@ -297,7 +302,8 @@ debugf("%s = '%s'", pstr, buf);
 	pstr = hash_get(pht, PMKCONF_BIN_INSTALL);
 	if (pstr != NULL) {
 	        /* append -c for compat with autoconf*/
-	        snprintf(buf, sizeof(buf), "%s -c", pstr);
+	        if (snprintf_b(buf, sizeof(buf), "%s -c", pstr) == false)
+			return(false);
 		if (hash_update_dup(pht, "INSTALL", buf) == HASH_ADD_FAIL)
 			return(false);
 	}
@@ -363,7 +369,9 @@ bool process_template(char *template, pmkdata *pgd) {
 	char	*plb,
 		*pbf,
 		*ptn,
+		*tptn,
 		*pfn,
+		*tpfn,
 		*ptmp,
 		 lbuf[MAXPATHLEN],
 		 buf[MAXPATHLEN],
@@ -376,11 +384,22 @@ bool process_template(char *template, pmkdata *pgd) {
 	pht = pgd->htab;
 
 	/* save template name */
-	ptn = strdup(basename(template));
+	tptn = basename(template);
+	if (tptn == NULL) {
+		errorf("couldn't extract the base portion of '%s' : %s.",
+			template, strerror(errno));
+		return(false);
+	}
+	ptn = strdup(tptn);
+	if (ptn == NULL) {
+		free(ptn);
+		errorf(ERRMSG_MEM);
+		return(false);
+	}	
 
 	/* get the file name */
-	if (strlcpy(lbuf, ptn, sizeof(lbuf)) >= sizeof(lbuf)) {
-		errorf("buffer overflow.");
+	if (strlcpy_b(lbuf, ptn, sizeof(lbuf)) == false) {
+		errorf(PMK_ERR_OVRFLOW);
 		return(false);
 	}
 
@@ -394,11 +413,27 @@ bool process_template(char *template, pmkdata *pgd) {
 	}
 
 	/* save generated file name */
-	pfn = strdup(basename(lbuf));
+	tpfn = basename(lbuf);
+	if (tpfn == NULL) {
+		errorf("couldn't extract the base portion of '%s' : %s.",
+			lbuf, strerror(errno));
+		return(false);
+	}
+	pfn = strdup(tpfn);
+	if (pfn == NULL) {
+		free(pfn);
+		errorf(ERRMSG_MEM);
+		return(false);
+	}
 
 	/* get relative path from srcdir */
 	/* NOTE : we use strdup to avoid problem with linux's dirname */
 	ptmp = strdup(template);
+	if (ptmp == NULL) {
+		free(ptmp);
+		errorf(ERRMSG_MEM);
+		return(false);
+	}
 	relpath(pgd->srcdir, dirname(ptmp), buf); /* XXX check ? */
 	free(ptmp);
 	/* apply to basedir */
@@ -415,7 +450,7 @@ bool process_template(char *template, pmkdata *pgd) {
 
 	tfd = fopen(template, "r");
 	if (tfd == NULL) {
-		errorf("cannot open '%s' : %s.", 
+		errorf("cannot open '%s' : %s.",
 			template, strerror(errno));
 		return(false);
 	}
@@ -423,7 +458,7 @@ bool process_template(char *template, pmkdata *pgd) {
 	dfd = fopen(fpath, "w");
 	if (dfd == NULL) {
 		fclose(tfd); /* close already opened tfd before leaving */
-		errorf("cannot open '%s' : %s.", 
+		errorf("cannot open '%s' : %s.",
 			fpath, strerror(errno));
 		return(false);
 	}
@@ -441,9 +476,14 @@ bool process_template(char *template, pmkdata *pgd) {
 			return(false); /* XXX error message ? */
 	}
 
-	snprintf(cibuf, sizeof(cibuf), "%s, generated from %s by PMK.", pfn, ptn);
+	if (snprintf_b(cibuf, sizeof(cibuf),
+				"%s generated from %s by PMK.",
+				pfn, ptn) == false)
+		return(false);
+
 	if (hash_update_dup(pht, "configure_input", cibuf) == HASH_ADD_FAIL)
 		return(false);
+
 	free(ptn);
 	free(pfn);
 
@@ -541,6 +581,7 @@ bool process_cmd(prsdata *pdata, pmkdata *pgd) {
 
 bool parse_cmdline(char **val, int nbval, pmkdata *pgd) {
 	bool	 rval = true;
+	char	*pstr;
 	htable	*ht;
 	int	 i;
 	prsopt	 opt;
@@ -552,7 +593,14 @@ bool parse_cmdline(char **val, int nbval, pmkdata *pgd) {
 		/* parse option */
 		rval = parse_clopt(val[i], &opt, PRS_PMKCONF_SEP);
 		if (rval == true) {
-			if (hash_update(ht, opt.key, opt.value) == HASH_ADD_FAIL) { /* no need to strdup */
+			pstr = po_get_str(opt.value);
+			if (pstr == NULL) {
+				errorf("unable to get value for %s.", opt.key);
+				return(false);
+			}
+
+			/* no need to strdup */
+			if (hash_update(ht, opt.key, pstr) == HASH_ADD_FAIL) {
 				errorf("%s.", PRS_ERR_HASH);
 				rval = false;
 			}
@@ -623,10 +671,10 @@ int main(int argc, char *argv[]) {
 			 nbpd,
 			 nbcd,
 			 ovrsw = 0,
-			 i,
 			 chr;
 	pmkdata		*pgd;
 	prsdata		*pdata;
+	unsigned int	 i;
 
 	/* get current path */
 	if (getcwd(buf, sizeof(buf)) == NULL) {
@@ -659,11 +707,19 @@ int main(int argc, char *argv[]) {
 				case 'e' :
 					/* enable switch(es) */
 					enable_sw = strdup(optarg);
+					if (enable_sw == NULL) {
+						errorf(ERRMSG_MEM);
+						exit(EXIT_FAILURE);
+					}
 					break;
 
 				case 'd' :
 					/* disable switch(es) */
 					disable_sw = strdup(optarg);
+					if (disable_sw == NULL) {
+						errorf(ERRMSG_MEM);
+						exit(EXIT_FAILURE);
+					}
 					break;
 
 				case 'f' :
@@ -674,10 +730,24 @@ int main(int argc, char *argv[]) {
 						exit(EXIT_FAILURE);
 					}
 
-					/* path of pmkfile is also the srcdir base */
-					/* NOTE : we use strdup to avoid problem with linux's dirname */
-					pstr = strdup(pgd->pmkfile); 
-					strlcpy(pgd->srcdir, dirname(pstr), sizeof(pgd->srcdir)); /* XXX check ??? */
+					/*
+						path of pmkfile is also the srcdir base
+
+						NOTE : we use strdup to avoid
+						problem with linux's dirname
+					*/
+					pstr = strdup(pgd->pmkfile);
+					if (pstr == NULL) {
+						free(pstr);
+						errorf(ERRMSG_MEM);
+						exit(EXIT_FAILURE);
+					}
+					if (strlcpy_b(pgd->srcdir, dirname(pstr),
+							sizeof(pgd->srcdir)) == false) {
+						free(pstr);
+						errorf(PMK_ERR_OVRFLOW);
+						exit(EXIT_FAILURE);
+					}
 					free(pstr);
 
 					pmkfile_set = true;
@@ -720,19 +790,36 @@ int main(int argc, char *argv[]) {
 
 	/* set basedir if needed */
 	if (basedir_set == false) {
-		strlcpy(pgd->basedir, buf, sizeof(pgd->basedir));
+		if (strlcpy_b(pgd->basedir, buf,
+					sizeof(pgd->basedir)) == false) {
+			errorf("failed to set 'basedir'.");
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	/* set pmkfile and srcdir if needed */
 	if (pmkfile_set == false) {
-		strlcpy(pgd->srcdir, buf, sizeof(pgd->srcdir));
+		if (strlcpy_b(pgd->srcdir, buf,
+					sizeof(pgd->srcdir)) == false) {
+			errorf("failed to set 'srcdir'.");
+			exit(EXIT_FAILURE);
+		}
+
 		abspath(pgd->srcdir, PREMAKE_FILENAME, pgd->pmkfile); /* should not fail */
 	}
 
 	if (buildlog == true) {
-		strlcpy(pgd->buildlog, PMK_BUILD_LOG, sizeof(pgd->buildlog));
+		if (strlcpy_b(pgd->buildlog, PMK_BUILD_LOG,
+					sizeof(pgd->buildlog)) == false) {
+			errorf(PMK_ERR_BLDLOG);
+			exit(EXIT_FAILURE);
+		}
 	} else {
-		strlcpy(pgd->buildlog, "/dev/null", sizeof(pgd->buildlog));
+		if (strlcpy_b(pgd->buildlog, "/dev/null",
+					sizeof(pgd->buildlog)) == false) {
+			errorf(PMK_ERR_BLDLOG);
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	/* init prsdata structure */
@@ -747,7 +834,8 @@ int main(int argc, char *argv[]) {
 		if (parse_pmkconf(fp, pgd->htab, PRS_PMKCONF_SEP, process_opt) == false) {
 			/* parsing failed */
 			clean(pgd);
-			errorf("failed to parse '%s'.", PREMAKE_CONFIG_PATH);
+			fclose(fp);
+			errorf(ERRMSG_PARSE, PREMAKE_CONFIG_PATH);
 			exit(EXIT_FAILURE);
 		}
 		fclose(fp);
@@ -755,7 +843,7 @@ int main(int argc, char *argv[]) {
 	} else {
 		/* configuration file not found */
 		clean(pgd);
-		errorf("cannot parse '%s', run pmksetup.", PREMAKE_CONFIG_PATH);
+		errorf(ERRMSG_PARSE " Run pmksetup(8).", PREMAKE_CONFIG_PATH);
 		exit(EXIT_FAILURE);
 	}
 
@@ -807,7 +895,8 @@ int main(int argc, char *argv[]) {
 			if (parse_pmkconf(fp, pgd->htab, PRS_PMKCONF_SEP, process_opt) == false) {
 				/* parsing failed */
 				clean(pgd);
-				errorf("failed to parse '%s'.", pgd->ovrfile);
+				fclose(fp);
+				errorf(ERRMSG_PARSE, pgd->ovrfile);
 				exit(EXIT_FAILURE);
 			}
 			fclose(fp);
@@ -835,9 +924,9 @@ int main(int argc, char *argv[]) {
 	}
 
 	/* open log file */
-	if (pmk_log_open(PREMAKE_LOG) == false) {
+	if (pmk_log_open(PMK_LOG) == false) {
 		clean(pgd);
-		errorf("while opening '%s'.", PREMAKE_LOG);
+		errorf("while opening '%s'.", PMK_LOG);
 		exit(EXIT_FAILURE);
 	}
 
@@ -858,7 +947,7 @@ int main(int argc, char *argv[]) {
 	fp = fopen(pgd->pmkfile, "r");
 	if (fp == NULL) {
 		clean(pgd);
-		errorf("cannot open '%s' : %s.", 
+		errorf("cannot open '%s' : %s.",
 			pgd->pmkfile, strerror(errno));
 		exit(EXIT_FAILURE);
 	}
