@@ -1,8 +1,8 @@
-/* $Id: pmkpc.c,v 1.12 2004/02/08 14:33:37 mipsator Exp $ */
+/* $Id: pmkpc.c,v 1.22 2004/12/26 23:42:12 mipsator Exp $ */
 
 /*
  * Copyright (c) 2004 Damien Couderc
- * Copyright (c) 2004 Xavier Santolaria
+ * Copyright (c) 2004 Xavier Santolaria <xavier@santolaria.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@
 #include "premake.h"
 
 
-/*#define DEBUG_PMKPC	1*/
+/*#define PMKPC_DEBUG	1*/
 
 extern char	*optarg;
 extern int	 optind;
@@ -83,7 +83,9 @@ pcopt	pcoptions[] = {
 size_t	nbpcopt = sizeof(pcoptions) / sizeof(pcopt);
 
 /*
-	XXX
+	init optcell structure
+
+	return : optcell structure
 */
 
 optcell *optcell_init(void) {
@@ -102,7 +104,11 @@ optcell *optcell_init(void) {
 }
 
 /*
-	XXX
+	destroy optcell structure
+
+	poc : optcell structure
+
+	return : -
 */
 
 void optcell_destroy(optcell *poc) {
@@ -110,10 +116,16 @@ void optcell_destroy(optcell *poc) {
 }
 
 /*
-	XXX
+	process command line options
+
+	ac : argument number
+	av : argument array
+	poc : optcell structure
+
+	return : boolean
 */
 
-bool pcgetopt(int ac, char *av[], optcell *poc) {
+bool pcgetopt(unsigned int ac, char *av[], optcell *poc) {
 	static char	 buf[PMKPC_MAX_OPT_SIZE];
 	char		*opt,
 			*pstr;
@@ -124,7 +136,7 @@ bool pcgetopt(int ac, char *av[], optcell *poc) {
 	idx = poc->idx;
 
 	if (idx >= ac) {
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{pcgetopt} ac = %d", ac);
 #endif
 		/* no more args */
@@ -134,7 +146,7 @@ debugf("{pcgetopt} ac = %d", ac);
 	opt = av[idx];
 	
 	if ((opt[0] != '-') || (opt[1] != '-')) {
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{pcgetopt} full opt = '%s'", opt);
 #endif
 		/* not an option */
@@ -143,7 +155,7 @@ debugf("{pcgetopt} full opt = '%s'", opt);
 	} else {
 		/* skip leading "--" */
 		opt = opt + 2;
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{pcgetopt} opt = '%s'", opt);
 #endif
 
@@ -151,7 +163,12 @@ debugf("{pcgetopt} opt = '%s'", opt);
 		pstr = strchr(opt, PMKPC_ARG_SEP);
 		if (pstr != NULL) {
 			/* argument found */
-			strlcpy(buf, opt, sizeof(buf)); /* XXX check ? */
+			if (strlcpy_b(buf, opt, sizeof(buf)) == false) {
+#ifdef PMKPC_DEBUG
+				debugf("{pcgetopt} strlcpy failed for '%s'", opt);
+#endif
+				return(false);
+			}
 
 			/* delimit option name */
 			buf[pstr - opt] = CHAR_EOS;
@@ -159,7 +176,7 @@ debugf("{pcgetopt} opt = '%s'", opt);
 			/* save argument position */
 			pstr++;
 			poc->arg = pstr;
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{pcgetopt} opt arg = '%s'", poc->arg);
 #endif
 
@@ -170,7 +187,7 @@ debugf("{pcgetopt} opt arg = '%s'", poc->arg);
 
 		l = strlen(opt);
 		idx++;
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{pcgetopt} idx = %d", idx);
 #endif
 	}
@@ -232,7 +249,13 @@ bool list_all(pkgdata *ppd) {
 	}
 
 	/* build format string to align descriptions */
-	snprintf(fmt, sizeof(fmt), "%%-%us %%s\n", maxs);
+	if (snprintf_b(fmt, sizeof(fmt), "%%-%us %%s\n",
+				(unsigned int)maxs) == false) {
+		/* cleaning */
+		hash_free_hkeys(phk);
+		free(descr);
+		return(false);
+	}
 
 	for(i = 0 ; i < phk->nkey ; i++) {
 		if (rval == true) {
@@ -272,11 +295,11 @@ void clean(pcdata *p_cd) {
 */
 
 void usage(void) {
-	int	i;
-	char	option[MAXPATHLEN];
-	size_t	alen,	/* alignement length */
-		olen,	/* option length */
-		cursor;
+	char		option[MAXPATHLEN];
+	size_t		alen,	/* alignement length */
+			olen,	/* option length */
+			cursor;
+	unsigned int	i;
 
 	alen = strlen(PMKPC_USAGE_ALIGN);
 
@@ -284,15 +307,21 @@ void usage(void) {
 	cursor = strlen(PMKPC_USAGE_STR);
 
 	for (i = 0; i < nbpcopt; i++) {
-		strlcpy(option, PMKPC_USAGE_OPEN_OPT, sizeof(option));
-		strlcat(option, pcoptions[i].name, sizeof(option));
+		strlcpy(option, PMKPC_USAGE_OPEN_OPT,
+					sizeof(option)); /* no check needed */
+		strlcat(option, pcoptions[i].name,
+					sizeof(option)); /* no check needed */
 
 		if (pcoptions[i].usagearg != NULL) {
-			strlcat(option, "=", sizeof(option));
-			strlcat(option, pcoptions[i].usagearg, sizeof(option)); 
+			strlcat(option, "=",
+					sizeof(option)); /* no check needed */
+			strlcat(option, pcoptions[i].usagearg,
+					sizeof(option)); /* no check needed */
 		}
 
-		strlcat(option, PMKPC_USAGE_CLOSE_OPT, sizeof(option));
+		strlcat(option, PMKPC_USAGE_CLOSE_OPT,
+					sizeof(option)); /* no check needed */
+
 		olen = strlen(option);
 		cursor += olen;
 
@@ -369,9 +398,9 @@ int main(int argc, char *argv[]) {
 		exit(EXIT_FAILURE);
 	}
 
-	while (poc->idx != argc) {
+	while (poc->idx != (unsigned int) argc) {
 		if (pcgetopt(argc, argv, poc) == true) {
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{main} id = %d", poc->id);
 #endif
 			/* found an option */
@@ -491,15 +520,21 @@ debugf("{main} id = %d", poc->id);
 						exit(EXIT_FAILURE);
 						break;
 					default :
-						errorf("unknow option -%c", 
+						errorf("unknown option -%c",
 							mod[1]);
 						exit(EXIT_SUCCESS);
 						break;
 				}
 			} else {
 				/* add new module */
-				da_push(gdata.pda, strdup(mod)); /* XXX check */
-#ifdef DEBUG_PMKPC
+				if (da_push(gdata.pda, strdup(mod)) == false) {
+					errorf("unable to store module name in dynarray.");
+#ifdef PMKPC_DEBUG
+					debugf("{main} mod = '%s'", mod);
+#endif
+					exit(EXIT_FAILURE);
+				}
+#ifdef PMKPC_DEBUG
 debugf("{main} new mod = '%s'", mod);
 #endif
 				poc->idx++;
@@ -530,9 +565,10 @@ debugf("{main} new mod = '%s'", mod);
 		if (parse_pmkconf(fp, gdata.pht, PRS_PMKCONF_SEP, process_opt) == false) {
 			/* parsing failed */
 			clean(&gdata);
+			fclose(fp);
 			errorf("failed to parse '%s'.", PREMAKE_CONFIG_PATH);
 			exit(EXIT_FAILURE);
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 		} else {
 debugf("{main} parsed '%s'", PREMAKE_CONFIG_PATH);
 #endif
@@ -540,15 +576,16 @@ debugf("{main} parsed '%s'", PREMAKE_CONFIG_PATH);
 		fclose(fp);
 	} else {
 		clean(&gdata);
-		errorf("cannot open '%s' : %s.", PREMAKE_CONFIG_PATH, strerror(errno));
+		errorf("cannot open '%s' : %s.",
+			PREMAKE_CONFIG_PATH, strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
 	/* try to get pkg-config lib path from pmk.conf */
 	pc_path = hash_get(gdata.pht, PMKCONF_PC_PATH_LIB);
 	if (pc_path == NULL) {
-		printf("Unable to find pkg-config libdir.\n");
 		clean(&gdata);
+		errorf("unable to find pkg-config libdir.");
 		exit(EXIT_FAILURE);
 	}
 
@@ -565,16 +602,16 @@ debugf("{main} parsed '%s'", PREMAKE_CONFIG_PATH);
 	}
 
 
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{main} usize = '%d'", da_usize(gdata.pda));
 #endif
 	for (i = 0 ; i < da_usize(gdata.pda) ; i++) {
 		mod = da_idx(gdata.pda, i);
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{main} mod = '%s' (i = %d)", mod, i);
 #endif
 		if (pkg_mod_exists(gdata.ppd, mod) == true) {
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{main} module '%s' found", mod);
 #endif
 			ppc = pkg_cell_add(gdata.ppd, mod); /* ppc is part of ppd, don't destroy */
@@ -651,7 +688,7 @@ debugf("{main} module '%s' found", mod);
 				printf("\n");
 			}
 		} else {
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("module not found");
 #endif
 
@@ -667,12 +704,16 @@ debugf("module not found");
 
 			/* set config tool filename */
 			if (cfgtcell_get_binary(pcd, mod, pc_cmd, sizeof(pc_cmd)) == false) {
-				snprintf(pc_cmd, sizeof(pc_cmd), "%s-config", mod); /* XXX check */
+				if (snprintf_b(pc_cmd, sizeof(pc_cmd),
+						"%s-config", mod) == false) {
+					errorf("failed to build configure tool name.");
+					exit(EXIT_FAILURE);
+				}
 			}
 
 			bpath = hash_get(gdata.pht, PMKCONF_PATH_BIN);
 			if (bpath == NULL) {
-				/* XXX */
+				errorf("unable to get binary path.");
 				exit(EXIT_FAILURE);
 			}
 
@@ -681,7 +722,7 @@ debugf("module not found");
 			/* looking for it in the path */
 			if (get_file_path(pc_cmd, bpath, pc_buf, sizeof(pc_buf)) == true) {
 				/* use CHECK_CONFIG */
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("Found alternative '%s' tool.", pc_cmd);
 #endif
 
@@ -744,16 +785,15 @@ debugf("Found alternative '%s' tool.", pc_cmd);
 			}
 		}
 	}
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("{main} destroy data");
 #endif
 	clean(&gdata);
 	if (pcd != NULL) {
-		/* XXX msg */
 		cfgtdata_destroy(pcd);
 	}
 
-#ifdef DEBUG_PMKPC
+#ifdef PMKPC_DEBUG
 debugf("End.");
 #endif
 
