@@ -1,4 +1,4 @@
-/* $Id: pmksetup.c,v 1.85 2004/02/12 14:11:33 mipsator Exp $ */
+/* $Id: pmksetup.c,v 1.110 2004/12/27 22:58:45 xsa Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -34,8 +34,10 @@
  *
  */
 
-#include <sys/stat.h>
-/* include it first as if it was <sys/types.h> - this will avoid errors */ 
+
+/*#define PMKSETUP_DEBUG 1*/
+
+/* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
 #include <sys/utsname.h>
 
@@ -48,16 +50,17 @@
 #include "compat/pmk_string.h"
 #include "compat/pmk_unistd.h"
 #include "common.h"
+#include "detect_cpu.h"
 #include "dynarray.h"
 #include "parse.h"
 #include "pmksetup.h"
 
 
-char	*__progname;		/* program name from argv[0] */
-
-int	 verbose_flag = 0;	/* -V option at the cmd-line */ 
-char	 sfn[MAXPATHLEN];	/* scratch file name */		
 FILE	*sfp;			/* scratch file pointer */
+char	sfn[MAXPATHLEN];	/* scratch file name */		
+
+htable	*ht;
+int	 verbose_flag = 0;	/* -V option at the cmd-line */
 
 hpair	predef[] = {
 		{PMKCONF_MISC_SYSCONFDIR,	SYSCONFDIR},
@@ -74,14 +77,17 @@ int	nbpredef = sizeof(predef) / sizeof(hpair);
  */
 int main(int argc, char *argv[]) {
 	FILE		*config;
-	bool		 process_clopts = false;
-	htable		*ht;
+	bool		 process_clopts = false,
+			 cfg_backup = false;
+	char		*pstr;
 	int		 ch,
 			 error = 0;
-	prsopt		 opt;
+	prsopt		 *ppo;
 	
-	extern int	 optind,
-			 errno;
+	extern int	 optind;
+#ifndef errno
+	extern int	 errno;
+#endif
 
 
 #ifndef PMK_USERMODE
@@ -92,9 +98,8 @@ int main(int argc, char *argv[]) {
 	}
 #endif
 
-	__progname = argv[0];
-
-	if ((ht = hash_init(MAX_CONF_OPT)) == NULL) {
+	ht = hash_init_adv(MAX_CONF_OPT, NULL, (void (*)(void *)) prsopt_destroy, NULL);
+	if (ht == NULL) {
 		errorf("cannot create hash table.");
 		exit(EXIT_FAILURE);
 	}
@@ -104,8 +109,8 @@ int main(int argc, char *argv[]) {
 		switch(ch) {
 			case 'r' :
 				/* mark to be deleted in hash */
-				if (hash_update_dup(ht, optarg, STR_FOR_REMOVE) == HASH_ADD_FAIL) {
-					errorf("hash update failed.");
+				if (record_data(ht, optarg, PMKSTP_REC_REMV, NULL) == false) {
+					errorf("failed to record '%s'.", optarg);
 					exit(EXIT_FAILURE);
 				}
 
@@ -113,17 +118,40 @@ int main(int argc, char *argv[]) {
 				break;
 
 			case 'u' :
-				if (parse_clopt(optarg, &opt, PRS_PMKCONF_SEP) == false) {
+				/* add or update value */
+				ppo = prsopt_init();
+				if (ppo == NULL) {
+					errorf("prsopt init failed.");
+					exit(EXIT_FAILURE);
+				}
+
+				if (parse_clopt(optarg, ppo, PRS_PMKCONF_SEP) == false) {
 					errorf("bad argument for -a option: %s.", optarg);
 					exit(EXIT_FAILURE);
 				}
 
 				/* add in hash */
-				if (hash_update_dup(ht, opt.key, opt.value) == HASH_ADD_FAIL) {
-					errorf("hash update failed.");
+				pstr = po_get_str(ppo->value);
+				if (pstr == NULL) {
+					errorf("failed to get argument for -a option");
 					exit(EXIT_FAILURE);
 				}
-				/* XXX TODO destroy data */
+
+				/* record prsopt structure directly, fast and simple */
+				ppo->opchar = PMKSTP_REC_UPDT;
+				if (hash_update(ht, ppo->key, ppo) == HASH_ADD_FAIL) {
+					prsopt_destroy(ppo);	
+					errorf("hash update failed.");	
+					exit(EXIT_FAILURE);		
+				}
+
+				/* XXX use record_data => cost much memory and cpu XXX            */
+				/*if (record_data(ht, ppo->key, PMKSTP_REC_UPDT, pstr) == false) {*/
+				/*        errorf("failed to record '%s'.", ppo->key);             */
+				/*        prsopt_destroy(ppo);                                    */
+				/*        exit(EXIT_FAILURE);                                     */
+				/*}                                                               */
+				/*prsopt_destroy(ppo);						  */
 
 				process_clopts = true;
 				break;
@@ -134,7 +162,7 @@ int main(int argc, char *argv[]) {
 				break;
 
 			case 'V' :
-				if (0 == verbose_flag)
+				if (verbose_flag == 0)
 					verbose_flag = 1;
 				break;
 
@@ -154,28 +182,31 @@ int main(int argc, char *argv[]) {
 	printf("\n\n");
 
 	/* check if syconfdir exists */
-	if (dir_exists(CONFDIR) == -1) {
+	if (dir_exists(CONFDIR) == false) {
 		verbosef("==> Creating '%s' directory.", CONFDIR);
 		if (mkdir(CONFDIR, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
-			errorf("cannot create '%s' directory : %s.", 
+			errorf("cannot create '%s' directory : %s.",
 				CONFDIR, strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 	}
 
 	if (process_clopts == false) {
-		/* standard behavior */
+		/* standard behavior, gathering data */
 		if (gather_data(ht) == false)
 			exit(EXIT_FAILURE);
 	}
 
-	if ((open_tmp_config() == -1))
+	if ((open_tmp_config() == false))
 		exit(EXIT_FAILURE);
 
 	/* parse configuration file */
-	if ((config = fopen(PREMAKE_CONFIG_PATH, "r")) != NULL) {
+	config = fopen(PREMAKE_CONFIG_PATH, "r");
+	if (config != NULL) {
+		/* switch backup flag */
+		cfg_backup = true;
+
 		printf("==> Configuration file found: %s\n", PREMAKE_CONFIG_PATH);
-		/* XXX FIXME TODO check ? */
 		if (parse_pmkconf(config, ht, PRS_PMKCONF_SEP, check_opt) == false) {
 			fclose(config);
 			errorf("parsing failed.");
@@ -184,7 +215,7 @@ int main(int argc, char *argv[]) {
 			fclose(config);
 		}
 	} else {
-		printf("==> Configuration file not found, generating one...\n");
+		printf("==> Configuration file not found.\n");
 	}
 
 	printf("==> Merging remaining data...\n");	
@@ -193,13 +224,24 @@ int main(int argc, char *argv[]) {
 	/* destroying the hash once we'r done with it */	
 	hash_destroy(ht);
 
-	if (close_tmp_config() < 0)
+	if (close_tmp_config() == false)
 		exit(EXIT_FAILURE);
 
-	if (error == 0)
+	if (cfg_backup == true) {
+		printf("==> Backing up configuration file: %s\n", PREMAKE_CONFIG_PATH_BAK);
+		if (rename(PREMAKE_CONFIG_PATH,PREMAKE_CONFIG_PATH_BAK) != 0) {
+			errorf("configuration file backup failed: %s.", strerror(errno));
+			error = -1;
+		}
+	}
+				
+	if (error == 0) {
 		/* copying the temporary config to the system one */
-		if (copy_config(sfn, PREMAKE_CONFIG_PATH) == -1)
+		printf("==> Saving configuration file: %s\n", PREMAKE_CONFIG_PATH);
+		/* XXX if (copy_config(sfn, PREMAKE_CONFIG_PATH) == false)*/
+		if (fcopy(sfn, PREMAKE_CONFIG_PATH, PREMAKE_CONFIG_MODE) == false)
 			exit(EXIT_FAILURE);
+	}
 
 #ifdef PMKSETUP_DEBUG
 	debugf("%s has not been deleted!", sfn);
@@ -207,17 +249,67 @@ int main(int argc, char *argv[]) {
 	if (unlink(sfn) == -1) {
 		errorf("cannot remove temporary file: '%s' : %s.",
 			sfn, strerror(errno));
-		exit(EXIT_FAILURE);	
+		error = -1;	
 	}
 #endif	/* PMKSETUP_DEBUG */
 
 	if (error != 0) {
-		errorf("returned code '%d' during parsing.", error);
-		exit(error);
+		exit(EXIT_FAILURE);	
 	}
+
 	return(0);
 }
 
+/*
+ *	record data
+ *
+ *	pht : hash table
+ *	key : record name
+ *	opchar : operation char
+ *		PMKSTP_REC_REMV => remove
+ *		PMKSTP_REC_UPDT => add/update
+ *	value : record data
+ *
+ *	return : true on success
+ */
+
+bool record_data(htable *pht, char *key, char opchar, char *value) {
+	prsopt	*ppo;
+
+	switch (opchar) {
+		case PMKSTP_REC_REMV :
+			/* delete record */
+			ppo = prsopt_init_adv(key, opchar, NULL);
+			if (ppo == NULL) {
+				return(false);
+			}
+			break;
+
+		case PMKSTP_REC_UPDT :
+			/* update record */
+			ppo = prsopt_init_adv(key, opchar, value);
+			if (ppo == NULL) {
+				return(false);
+			}
+			break;
+
+		default :
+			/* unknown operation */
+#ifdef PMKSETUP_DEBUG
+	debugf("unknown '%c' operator in record_data.", opchar);
+#endif
+			return(false);
+	}
+
+	if (hash_update(pht, ppo->key, ppo) == HASH_ADD_FAIL) {
+		prsopt_destroy(ppo);
+		return(false);
+	}
+
+	return(true);
+}
+
+
 /*
  * gathering of system data, predefinied data, ...
  */
@@ -226,23 +318,30 @@ bool gather_data(htable *pht) {
 	printf("==> Looking for default parameters...\n");
 
 	/* gather env variables and look for specific binaries */
-	if ((get_env_vars(pht) == -1) || (get_binaries(pht) == -1))
-		return(false);
+	if ((get_env_vars(pht) == false) || (get_binaries(pht) == false))
+		return(false);	/* XXX error messages ? */
 
 	/* set predifined variables */
-	if (predef_vars(pht) == -1) {
-		errorf("predefined variables."); 
+	if (predef_vars(pht) == false) {
+		errorf("predefined variables.");
 		return(false);
 	}
 
 	check_libpath(pht);
 
+	/* check byte order */
 	if (byte_order_check(pht) == false) {
 		errorf("failure in byte order check.");
 		return(false);
 	}
 
-	if (check_echo(pht) == -1) {
+	/* try to detect cpu family and specific data */
+	if (get_cpu_data(pht) == false) {
+		errorf("failure in cpu detection.");
+		return(false);
+	}
+
+	if (check_echo(pht) == false) {
 		errorf("failure in echo check.");
 		return(false);
 	}
@@ -253,26 +352,48 @@ bool gather_data(htable *pht) {
 /*
  * write remaining data
  *
- * 	ht: hash table
+ * 	pht: hash table
  */
-void write_new_data(htable *ht) {
-	char	*val;
-	int	 i;
-	hkeys	*phk;
+void write_new_data(htable *pht) {
+	char		*val;
+	hkeys		*phk;
+	prsopt		*ppo;
+	unsigned int	 i;
 
-	phk = hash_keys(ht);
+	phk = hash_keys(pht);
 	if (phk != NULL) {
 		/* sort hash table */
 		qsort(phk->keys, phk->nkey, sizeof(char *), keycomp);
 
 		/* processing remaining keys */
 		for(i = 0 ; i < phk->nkey ; i++) {
-			val = (char *) hash_get(ht, phk->keys[i]);
-
-			/* check if this key is marked for being removed */
-			if (strncmp(val, STR_FOR_REMOVE, sizeof(STR_FOR_REMOVE)) != 0) {
-				/* add value */
-				fprintf(sfp, PMKSTP_WRITE_FORMAT, phk->keys[i], CHAR_ASSIGN_UPDATE, val);
+#ifdef PMKSETUP_DEBUG
+	debugf("write_new_data: processing '%s'", phk->keys[i]);
+#endif
+			ppo = hash_get(pht, phk->keys[i]);
+			if (ppo == NULL) {
+#ifdef PMKSETUP_DEBUG
+				debugf("write_new_data: unable to get '%s' record.", phk->keys[i]);
+#endif
+				/* XXX  exit ? */
+			} else {
+				/* check if this key is marked for being removed */
+				if (ppo->opchar != PMKSTP_REC_REMV) {
+					/* add value */
+					val = po_get_str(ppo->value);
+					if (val != NULL) {
+						fprintf(sfp, PMKSTP_WRITE_FORMAT, phk->keys[i], CHAR_ASSIGN_UPDATE, val);
+#ifdef PMKSETUP_DEBUG
+					} else {
+						debugf("write_new_data: skipping '%s', empty value.", phk->keys[i]);
+#endif
+						/* XXX  exit ? */
+					}
+#ifdef PMKSETUP_DEBUG
+				} else {
+					debugf("write_new_data: skipping '%s', remove record.", phk->keys[i]);
+#endif
+				}
 			}
 		}
 
@@ -294,7 +415,7 @@ void write_new_data(htable *ht) {
  *
  *	returns an integer greater than, equal to, or
  *		less than 0 according as the character a is
- *		greather, equal to, or less than the character b. 
+ *		greather, equal to, or less than the character b.
  */
 int keycomp(const void *a, const void *b) {
 	return(strcmp(*(const char **)a, *(const char **)b));
@@ -309,47 +430,78 @@ int keycomp(const void *a, const void *b) {
  *	return : boolean
  */
 
-bool check_opt(htable *pht, prsopt *popt) {
+bool check_opt(htable *cht, prsopt *popt) {
 	char	*recval,
 		*optval;
+	prsopt	*ppo;
 
 	optval = po_get_str(popt->value);
 
 	if ((popt->opchar == CHAR_COMMENT) || (popt->opchar == CHAR_EOS)) {
 		/* found comment or empty line */
 		fprintf(sfp, "%s\n", optval);
+#ifdef PMKSETUP_DEBUG
+		debugf("check_opt() appended '%s\n'", optval);
+#endif
 		return(true);
 	}
 
-	recval = (char *) hash_get(pht, popt->key);
-	if (recval != NULL) {
+#ifdef PMKSETUP_DEBUG
+	debugf("check_opt: checking key '%s'", popt->key);
+#endif
+
+	ppo = hash_get(ht, popt->key);
+	/*if ((ppo != NULL) && (ppo->value != NULL)) {*//* on remove value is NULL !!! */
+	if (ppo != NULL) {
+		/* XXX check value */
+		recval = po_get_str(ppo->value);
+
+#ifdef PMKSETUP_DEBUG
+		debugf("check_opt: original opchar '%c'", popt->opchar);
+#endif
+
 		/* checking the VAR<->VALUE separator */
 		switch (popt->opchar) {
 			case CHAR_ASSIGN_UPDATE :
-				/* check if this key is marked for being removed */
-				if (strncmp(recval, STR_FOR_REMOVE, sizeof(STR_FOR_REMOVE)) != 0) {
-					/* update value */
-					fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_UPDATE, recval);
-				} else {
-					verbosef("Removing '%s'", popt->key);
-				}
+#ifdef PMKSETUP_DEBUG
+				debugf("check_opt: update opchar '%c'", ppo->opchar);
+#endif
+				switch (ppo->opchar) {
+					case PMKSTP_REC_REMV :
+						/* do nothing, value will not be refreshed */
+						verbosef("Removing '%s'", popt->key);
+						break;
+					case PMKSTP_REC_UPDT :
+						/* update value */
+						fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_UPDATE, recval);
+#ifdef PMKSETUP_DEBUG
+						debugf("check_opt() appended '" PMKSTP_WRITE_FORMAT "'", popt->key, CHAR_ASSIGN_UPDATE, recval);
+#endif
+						break;
 
-				hash_delete(pht, popt->key);
+				}
+				hash_delete(cht, popt->key);
 				break;
 
 			case CHAR_ASSIGN_STATIC :
-				/* static definition, stay unchanged */ 
+				/* static definition, stay unchanged */
 				fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_STATIC, optval);
-				hash_delete(pht, popt->key);
+#ifdef PMKSETUP_DEBUG
+				debugf("check_opt() appended '" PMKSTP_WRITE_FORMAT "'", popt->key, CHAR_ASSIGN_STATIC, optval);
+#endif
+				hash_delete(cht, popt->key);
 				break;
 
-				default :
-					/* should not happen now */
-					errorf("unknown operator '%c'.", popt->opchar);
-					return(false);
-					break;
+			default :
+				/* should not happen now */
+				errorf("unknown operator '%c'.", popt->opchar);
+				return(false);
+				break;
 		}
 	} else {
+#ifdef PMKSETUP_DEBUG
+	debugf("check_opt: key '%s' does not exist.", popt->key);
+#endif
 		fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, popt->opchar, optval);
 	}
 
@@ -359,30 +511,31 @@ bool check_opt(htable *pht, prsopt *popt) {
 /*
  * Open temporary configuration file
  *
- *	returns:  0 on success
- *		 -1 on failure
+ *	return:  boolean
  */
-int open_tmp_config(void) {
-	int	fd = -1;
+bool open_tmp_config(void) {
+	int	fd;
 
 	/* creating temporary file to build new configuration file */
-	snprintf(sfn, sizeof(sfn), PREMAKE_CONFIG_TMP);
+	if (snprintf_b(sfn, sizeof(sfn), PREMAKE_CONFIG_TMP) == false)
+		return(false);
 
-	if ((fd = mkstemp(sfn)) == -1) {
-		errorf("could not create temporary file '%s' : %s.", 
+	fd = mkstemp(sfn);
+	if (fd == -1) {
+		errorf("could not create temporary file '%s' : %s.",
 			sfn, strerror(errno));
-		return(-1);
+		return(false);
 	}
-	if ((sfp = fdopen(fd, "w")) == NULL) {
-		if (fd != -1) {
-			unlink(sfn);
-			close(fd);
-		}	
-		errorf("cannot open temporary file '%s' : %s.", 
+
+	sfp = fdopen(fd, "w");
+	if (sfp == NULL) {
+		unlink(sfn);
+		close(fd);
+		errorf("cannot open temporary file '%s' : %s.",
 			sfn, strerror(errno));
-		return(-1);
+		return(false);
 	}
-	return(0);
+	return(true);
 }
 
 
@@ -392,19 +545,20 @@ int open_tmp_config(void) {
  *	it deletes the temporary file on exit except if the
  *	PMKSETUP_DEBUG mode is on.
  *
- *	returns:  0 on success
- *		 -1 on failure 
+ *	return:  boolean
  */
-int close_tmp_config(void) {
-	if (sfp) {
-		if (fclose(sfp) < 0) {
-			errorf("cannot close temporary file '%s' : %s.", 
-				sfp, strerror(errno));
-			return(-1);
-		}
-		sfp = NULL;
-	}        
-	return(0);
+bool close_tmp_config(void) {
+	if (sfp == NULL)
+		return(true);
+
+	if (fclose(sfp) != 0) {
+		errorf("cannot close temporary file '%s' : %s.",
+			sfp, strerror(errno));
+		return(false);
+	}
+
+	sfp = NULL; /* XXX useful ? */
+	return(true);
 }
 
 
@@ -413,99 +567,111 @@ int close_tmp_config(void) {
  *
  *	ht: hash table where we have to store the values
  *
- *	returns:  0 on success
- *		 -1 on failure
+ *	return:  boolean
  */
-int get_env_vars(htable *ht) {
+bool get_env_vars(htable *pht) {
 	struct utsname	 utsname;
 	char		*bin_path;
 
 	if (uname(&utsname) == -1) {
 		errorf("uname : %s.", strerror(errno));
-		return(-1);
+		return(false);
 	}
 
-	if (hash_update_dup(ht, PMKCONF_OS_NAME, utsname.sysname) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_OS_NAME, 'u', utsname.sysname) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_OS_NAME, utsname.sysname);
 
-	if (hash_update_dup(ht, PMKCONF_OS_VERS, utsname.release) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_OS_VERS, 'u', utsname.release) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_OS_VERS, utsname.release);	
 	
-	if (hash_update_dup(ht, PMKCONF_OS_ARCH, utsname.machine) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_OS_ARCH, 'u', utsname.machine) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_OS_ARCH, utsname.machine);
 
-
 	/* getting the environment variable PATH */
 	if ((bin_path = getenv("PATH")) == NULL) {
 		errorf("could not get the PATH environment variable.");
-		return(-1);
+		return(false);
 	}
 
-	/* 
-	 * replace the string separator in the PATH 
+	/*
+	 * replace the string separator in the PATH
 	 * to fit to our standards
 	 */
 	char_replace(bin_path, PATH_STR_DELIMITER, CHAR_LIST_SEPARATOR);
 
-	if (hash_update_dup(ht, PMKCONF_PATH_BIN, bin_path) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_PATH_BIN, 'u', bin_path) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_PATH_BIN, bin_path);
 
-	return(0);
+	return(true);
 }
 
 
-/* 
+/*
  * Look for location of some predefined binaries
  *
  *	ht: hash table where we have to store the values
  *
- *	returns:  0 on success
- *		 -1 on failure
- */ 
-int get_binaries(htable *ht) {
-	char	 fbin[MAXPATHLEN];	/* full binary path */
-	dynary	*stpath;
-	int	 i;
+ *	return: boolean
+ */
+bool get_binaries(htable *pht) {
+	char		 fbin[MAXPATHLEN],	/* full binary path */
+			*pstr;
+	dynary		*stpath;
+	prsopt		*ppo;
+	unsigned int	 i;
 
         /*
-         * splitting the PATH variable and storing in a 
+         * splitting the PATH variable and storing in a
          * dynamic array for later use by find_file
          */
-	stpath = str_to_dynary((char *) hash_get(ht, PMKCONF_PATH_BIN), CHAR_LIST_SEPARATOR);
+
+	ppo = hash_get(pht, PMKCONF_PATH_BIN);
+	if (ppo == NULL) {
+		errorf("unable to get binary path record.");
+		return(false);
+	}
+
+	pstr = po_get_str(ppo->value);
+	if (pstr == NULL) {
+		/* XXX error */
+		return(false);
+	}
+
+	stpath = str_to_dynary(pstr , CHAR_LIST_SEPARATOR);
 	if (stpath == NULL) {
 		errorf("could not split the PATH environment variable correctly.");
-		return(-1);	
+		return(false);	
 	}
 
 	if (find_file(stpath, "cc", fbin, sizeof(fbin)) == true) {
-		if (hash_update_dup(ht, "BIN_CC", fbin) == HASH_ADD_FAIL) {
+		if (record_data(pht, PMKCONF_BIN_CC, 'u', fbin) == false) {
 			da_destroy(stpath);
-			return(-1);
+			return(false);
 		}
-		verbosef("Setting '%s' => '%s'", "BIN_CC", fbin);
+		verbosef("Setting '%s' => '%s'", PMKCONF_BIN_CC, fbin);
 	} else {
 		if (find_file(stpath, "gcc", fbin, sizeof(fbin)) == true) {
-			if (hash_update_dup(ht, "BIN_CC", fbin) == HASH_ADD_FAIL) {
+			if (record_data(pht, PMKCONF_BIN_CC, 'u', fbin) == false) {
 				da_destroy(stpath);
-				return(-1);
+				return(false);
 			}
-			verbosef("Setting '%s' => '%s'", "BIN_CC", fbin);
+			verbosef("Setting '%s' => '%s'", PMKCONF_BIN_CC, fbin);
 		} else {
 			errorf("cannot find a C compiler.");
 			da_destroy(stpath);
-			return(-1);
+			return(false);
 		}
 	}
 
 	for (i = 0; i < MAXBINS; i++) {
 		if (find_file(stpath, binaries[i][0], fbin, sizeof(fbin)) == true) {
-			if (hash_update_dup(ht, binaries[i][1], fbin) == HASH_ADD_FAIL) {
+			if (record_data(pht, binaries[i][1], 'u', fbin) == false) {
 				da_destroy(stpath);
-				return(-1);
+				return(false);
 			}
 			verbosef("Setting '%s' => '%s'", binaries[i][1], fbin);
 		} else {
@@ -513,7 +679,7 @@ int get_binaries(htable *ht) {
 		}
 	}
 	da_destroy(stpath);
-	return(0);
+	return(true);
 }
 
 
@@ -523,41 +689,39 @@ int get_binaries(htable *ht) {
  *
  *	pht: hash table where we have to store the values
  *
- *	returns:  0 on success
- *		 -1 on failure
- */ 
-int predef_vars(htable *pht) {
+ *	returns:  boolean
+ */
+bool predef_vars(htable *pht) {
 	int	i;
 
 	for (i = 0 ; i < nbpredef ; i++) {
-		verbosef("Setting '%s' => '%s'", predef[i].key, predef[i].value);
-		if (hash_update_dup(pht, predef[i].key, predef[i].value) == HASH_ADD_FAIL) {
+		if (record_data(pht, predef[i].key, 'u', predef[i].value) == false) {
 			errorf("failed to add '%s'.", predef[i].key);
-			return(-1);
+			return(false);
 		}
+		verbosef("Setting '%s' => '%s'", predef[i].key, predef[i].value);
 	}
 
-	return(0);
+	return(true);
 }
 
 /*
  *	pht: hash table where we have to store the values
  *
- *	returns:  0 on success
- *		 -1 on failure
+ *	returns:  boolean
  */
-int check_echo(htable *pht) {
+bool check_echo(htable *pht) {
 	FILE	*echo_pipe = NULL;
 	char	buf[TMP_BUF_LEN],
-		echocmd[MAXPATHLEN];
+		echocmd[TMP_BUF_LEN];
 	char	*echo_n, *echo_c, *echo_t;
 	size_t	s;
 
-	snprintf(echocmd, sizeof(echocmd), ECHO_CMD);
+	snprintf(echocmd, sizeof(echocmd), ECHO_CMD); /* should not fail */
 
 	if ((echo_pipe = popen(echocmd, "r")) == NULL) {
 		errorf("unable to execute '%s'.", echocmd);
-		return(-1);
+		return(false);
 	}
 
 	s = fread(buf, sizeof(char), sizeof(buf), echo_pipe);
@@ -566,7 +730,7 @@ int check_echo(htable *pht) {
 	if (feof(echo_pipe) == 0) {
 		errorf("pipe not empty.");
 		pclose(echo_pipe);
-		return(-1);
+		return(false);
 	}
 
 	pclose(echo_pipe);
@@ -596,85 +760,164 @@ int check_echo(htable *pht) {
 					echo_t = ECHO_EMPTY;
 				} else {
 					errorf("unable to set ECHO_* variables.");
-					return(-1);
+					return(false);
 				}
 			}
 		}
 	}
 
-	if (hash_update_dup(pht, PMKCONF_AC_ECHO_N, echo_n) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_AC_ECHO_N, 'u', echo_n) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_AC_ECHO_N, echo_n);
 	
-	if (hash_update_dup(pht, PMKCONF_AC_ECHO_C, echo_c) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_AC_ECHO_C, 'u', echo_c) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_AC_ECHO_C, echo_c);
 
-	if (hash_update_dup(pht, PMKCONF_AC_ECHO_T, echo_t) == HASH_ADD_FAIL)
-		return(-1);
+	if (record_data(pht, PMKCONF_AC_ECHO_T, 'u', echo_t) == false)
+		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_AC_ECHO_T, echo_t);
 
-	return(0);
+	return(true);
 }
 
 
 /*
+ *	check pkgconfig libpath
+ *
  *	pht: hash table where we have to store the values
  *
- *	returns:  0 on success
- *		 -1 on failure
+ *	returns:  boolean
+ *
+ *	NOTE: this path must be relative to the real prefix. This means
+ *		that we can't rely on predefined prefix.
  */
 
-int check_libpath(htable *pht) {
-	char	libpath[MAXPATHLEN];
+bool check_libpath(htable *pht) {
+	char	 libpath[MAXPATHLEN];
 
-	strlcpy(libpath, hash_get(pht, PMKCONF_MISC_PREFIX), sizeof(libpath));
-	strlcat(libpath, PMKVAL_LIB_PKGCONFIG, sizeof(libpath));
+	/* build the path dynamically */
+	/* variable prefix */
+	strlcpy(libpath, "$", sizeof(libpath)); /* should not fail */
+
+	/* prefix variable name */
+	strlcat(libpath, PMKCONF_MISC_PREFIX,
+				sizeof(libpath)); /* no check yet */
+
+	/* pkgconfig path suffix */
+	if (strlcat_b(libpath, PMKVAL_LIB_PKGCONFIG,
+				sizeof(libpath)) == false)
+		return(false);
 
 	if (hash_get(pht, PMKCONF_BIN_PKGCONFIG) != NULL) {
 		if (dir_exists(libpath) == 0) {
-			if (hash_update_dup(pht, PMKCONF_PC_PATH_LIB, libpath) == HASH_ADD_FAIL)
-				return(-1);
-			verbosef("Setting '%s' => '%s'", PMKCONF_PC_PATH_LIB, libpath);
+			if (record_data(pht, PMKCONF_PC_PATH_LIB,
+						'u', libpath) == false)
+				return(false);
+
+			verbosef("Setting '%s' => '%s'",
+					PMKCONF_PC_PATH_LIB, libpath);
 		} else {
 			verbosef("**warning: %s does not exist.", libpath);
 		}
 	}
-	return(0);
+	return(true);
 }
 
 /*
- *	check if a directory does exist 
+ *	check the cpu family 
+ *
+ *	uname_m : uname machine string 
+ *
+ *	returns:  cpu family string or NULL
+ */
+
+bool get_cpu_data(htable *pht) {
+	char		*uname_m;
+	char		*pstr;
+	hkeys		*phk;
+	htable		*spht;
+	pmkobj		*po;
+	prsdata		*pdata;
+	prsopt		*ppo;
+	unsigned int	 i;
+
+
+	ppo = hash_get(pht, PMKCONF_OS_ARCH);
+	if (ppo == NULL)
+		return(false);
+
+	po = ppo->value;
+	if (po == NULL)
+		return(false);
+
+	uname_m = po_get_str(po);
+	if (uname_m == NULL)
+		return(false);
+
+	pdata = parse_cpu_data(PMKCPU_DATA);
+	if (pdata == NULL) {
+		return(false);
+	}
+
+	pstr = check_cpu_arch(uname_m, pdata); /* no check, never NULL */
+	if (record_data(pht, PMKCONF_HW_CPU_ARCH, 'u', pstr) == false)
+		return(false);
+	verbosef("Setting '%s' => '%s'", PMKCONF_HW_CPU_ARCH, pstr);
+
+	spht = arch_wrapper(pdata, pstr);
+	if (spht != NULL) {
+		phk = hash_keys(spht);
+		if (phk != NULL) {
+			for(i = 0 ; i < phk->nkey ; i++) {
+				pstr = hash_get(spht, phk->keys[i]); /* XXX check ? */
+
+				if (record_data(pht, phk->keys[i], 'u', pstr) == false)
+					return(false);
+				verbosef("Setting '%s' => '%s'", phk->keys[i], pstr);
+			}
+
+			hash_free_hkeys(phk);
+		}
+		hash_destroy(spht);
+	}
+
+	prsdata_destroy(pdata);
+
+	return(true);
+}
+
+/*
+ *	check if a directory does exist
  *
  *	fdir : directory to search
  *
- *	returns:  0 on success
- *		 -1 on failure 
+ *	returns:  boolean
  */
 
-int dir_exists(const char *fdir) {
+bool dir_exists(const char *fdir) {
         DIR     *dirp;
         size_t  len;
-        
+
 	len = strlen(fdir);
 
 	if (len < MAXPATHLEN) {
 		dirp = opendir(fdir);
 		if (dirp != NULL) {
 			closedir(dirp);
-			return(0);
+			return(true);
 		}
 	}
-	return(-1);
+	return(false);
 }
 
 /*
-	byte order check
-
-	pht : hash table to store value
-
-	return : boolean value
-*/
+ *	byte order check
+ *
+ *	pht : hash table to store value
+ *
+ *	return : boolean value
+ */
 
 bool byte_order_check(htable *pht) {
 	char	bo_type[16];
@@ -682,17 +925,22 @@ bool byte_order_check(htable *pht) {
 
 	if (((((char *)&num)[0]) == 0x41) && ((((char *)&num)[1]) == 0x42) &&
 	    ((((char *)&num)[2]) == 0x43) && ((((char *)&num)[3]) == 0x44)) {
-		strlcpy(bo_type, HW_ENDIAN_BIG, sizeof(bo_type));
+		strlcpy(bo_type, HW_ENDIAN_BIG,
+				sizeof(bo_type)); /* should not fail */
 	} else {
-		if ( ((((char *)&num)[3]) == 0x41) && ((((char *)&num)[2]) == 0x42) && 
-		    ((((char *)&num)[1]) == 0x43) && ((((char *)&num)[0]) == 0x44) ) {
-			strlcpy(bo_type, HW_ENDIAN_LITTLE, sizeof(bo_type));
+		if (((((char *)&num)[3]) == 0x41) &&
+				((((char *)&num)[2]) == 0x42) &&
+				((((char *)&num)[1]) == 0x43) &&
+				((((char *)&num)[0]) == 0x44)) {
+			strlcpy(bo_type, HW_ENDIAN_LITTLE,
+					sizeof(bo_type)); /* should not fail */
 		} else {
-			strlcpy(bo_type, HW_ENDIAN_UNKNOWN, sizeof(bo_type));
+			strlcpy(bo_type, HW_ENDIAN_UNKNOWN,
+					sizeof(bo_type)); /* should not fail */
 		}
 	}
 
-	if (hash_update_dup(pht, PMKCONF_HW_BYTEORDER, bo_type) == HASH_ADD_FAIL)
+	if (record_data(pht, PMKCONF_HW_BYTEORDER, 'u', bo_type) == false)
 		return(false);
 
 	verbosef("Setting '%s' => '%s'", PMKCONF_HW_BYTEORDER, bo_type);
@@ -701,61 +949,13 @@ bool byte_order_check(htable *pht) {
 }
 
 
-/*
- * Copy the temporary configuration file to the system one
- *
- *	tmp_config: temporary configuration file name
- *	config: system configuration file name
- *
- *	returns:  0 on success
- *		 -1 on failure  	
- */
-int copy_config(const char *tmp_config, const char *config) {
-	FILE	*fp_t,
-		*fp_c;
-	int	 rval;
-	char	 buf[MAX_LINE_BUF];
-
-
-	if ((fp_t = fopen(tmp_config, "r")) == NULL) {
-		errorf("cannot open temporary configuration "
-			"file for reading '%s' : %s.", tmp_config, 
-				strerror(errno));
-		return(-1);	
-	}
-	if ((fp_c = fopen(config, "w")) == NULL) {
-		errorf("cannot open '%s' for writing : %s.", 
-			config, strerror(errno));
-
-		fclose(fp_t);	
-		return(-1);
-	}
-
-	while(get_line(fp_t, buf, MAX_LINE_BUF) == true) {
-		fprintf(fp_c, "%s\n", buf);
-	}
-
-	if (feof(fp_t) == 0) {
-		errorf("read failure, cannot copy "
-			"'%s' to '%s'.", tmp_config, config);
-		rval = -1;	
-	} else
-		rval = 0;
-
-	fclose(fp_t);
-	fclose(fp_c);
-
-	return(rval);
-}
-
-
 /*
  * Replace a character in a string
  *
  *	buf: string we want to modify
  *	search: the char with want to replace
  *	replace: the new char which will replace 'search'
- */ 
+ */
 void char_replace(char *buf, const char search, const char replace) {
 	char	c;
 	int	i = 0;
@@ -790,15 +990,8 @@ void verbosef(const char *fmt, ...) {
  * pmksetup(8) usage
  */
 void usage(void) {
-	/*fprintf(stderr, "Usage: %s [-hvV] [-a var=val] "*/
-	fprintf(stderr, "Usage: %s [-hvV] "
-		"[-r var] [-u var=val]\n", __progname);
-	/*fprintf(stderr, "\t-a var=val   Append to a variable in pmk.conf\n");*/
-	fprintf(stderr, "\t-r var       Remove a variable in pmk.conf\n");
-	fprintf(stderr, "\t-u var=val   Update a variable in pmk.conf\n");
-	fprintf(stderr, "\t-h           Display this help menu\n"); 
-	fprintf(stderr, "\t-v           Display version number\n");
-	fprintf(stderr, "\t-V           Verbose, display debugging messages\n");
+	fprintf(stderr, "usage: pmksetup [-hVv] "
+		"[-r variable] [-u variable=value]\n");
 	exit(EXIT_FAILURE);
 }
 
