@@ -1,4 +1,4 @@
-/* $Id: premake.h,v 1.91 2004/03/14 17:34:36 mipsator Exp $ */
+/* $Id: premake.h,v 1.113 2004/12/11 19:40:00 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -38,19 +38,20 @@
 #define _PREMAKE_H_
 
 #include "compat/pmk_stdbool.h"
+#include "errmsg.h"
 #include "pmk_obj.h"
 
 /* We include sys/param.h to get MAXPATHLEN.
-   This is known to work under following operanding systems :
+   This is known to work under following operating systems:
 	OpenBSD
 	FreeBSD
 	NetBSD
-	MACOSX
-	Solaris			(not verified)
+	Mac OS X
+	Solaris
 	SunOS
 	HP-UX
-	AIX			(not verified)
-	IRIX			(not verified)
+	AIX
+	IRIX
 	OSF1
 	Ultrix			(not verified)
 	Linux based systems	(Debian, Mandrake, RedHat, Slackware, Suse)
@@ -62,7 +63,7 @@
 
 
    Comments about this stuff is welcome. If your system is not
-   supported then take contact with us to fix it.
+   supported then get in touch with us to fix it.
 */
 #include <sys/param.h>
 #ifndef MAXPATHLEN
@@ -105,7 +106,7 @@
 #endif
 
 /* only used for snapshots, comment for release */
-/*#define PREMAKE_SNAP		"1"*/
+#define PREMAKE_SNAP		"9"
 
 /* build version string */
 #ifndef PREMAKE_SNAP
@@ -117,9 +118,9 @@
 #define PREMAKE_CONFIG		"pmk.conf"
 #define PREMAKE_FILENAME	"pmkfile"
 #define PREMAKE_PKGFILE		"pkgfile"
-#define PREMAKE_LOG		"pmk.log"
 
-#define PREMAKE_CONFIG_PATH	CONFDIR "/" PREMAKE_CONFIG 
+#define PREMAKE_CONFIG_PATH	CONFDIR "/" PREMAKE_CONFIG
+#define PREMAKE_CONFIG_PATH_BAK	PREMAKE_CONFIG_PATH ".bak"
 
 #define PREMAKE_TMP_DIR		TMPDIR
 
@@ -129,20 +130,28 @@
 
 /* key names (see also pmksetup.h) */
 #define PMKCONF_BIN_AR		"BIN_AR"
+#define PMKCONF_BIN_AS		"BIN_AS"
 #define PMKCONF_BIN_AWK		"BIN_AWK"
 #define PMKCONF_BIN_CAT		"BIN_CAT"
 #define PMKCONF_BIN_CC		"BIN_CC"
 #define PMKCONF_BIN_CPP		"BIN_CPP"
 #define PMKCONF_BIN_CXX		"BIN_CXX"
+#define PMKCONF_BIN_ECHO	"BIN_ECHO"
 #define PMKCONF_BIN_EGREP	"BIN_EGREP"
 #define PMKCONF_BIN_GREP	"BIN_GREP"
+#define PMKCONF_BIN_ID		"BIN_ID"
 #define PMKCONF_BIN_INSTALL	"BIN_INSTALL"
+#define PMKCONF_BIN_LEX		"BIN_LEX"
+#define PMKCONF_BIN_LN		"BIN_LN"
 #define PMKCONF_BIN_PKGCONFIG	"BIN_PKGCONFIG"
 #define PMKCONF_BIN_RANLIB	"BIN_RANLIB"
 #define PMKCONF_BIN_SH		"BIN_SH"
 #define PMKCONF_BIN_STRIP	"BIN_STRIP"
+#define PMKCONF_BIN_SUDO	"BIN_SUDO"
 #define PMKCONF_BIN_TAR		"BIN_TAR"
+#define PMKCONF_BIN_YACC	"BIN_YACC"
 
+#define PMKCONF_HW_CPU_ARCH	"HW_CPU_ARCH"
 #define PMKCONF_HW_BYTEORDER	"HW_BYTEORDER"
 
 #define PMKCONF_PATH_BIN	"PATH_BIN"
@@ -167,12 +176,18 @@
 #define PMKVAL_BIN_PKGCONFIG	"pkg-config"
 #define PMKVAL_LIB_PKGCONFIG	"/lib/pkgconfig" /* preceeded by PREFIX */
 
+#define PMKVAL_ENV_LIBS		"LIBS"
+#define PMKVAL_ENV_CFLAGS	"CFLAGS"
+
 /* prefix character used for comments */
 #define CHAR_COMMENT		'#'
 
 /* end of string */
 #define CHAR_EOS		'\0'
 
+/* carriage return */
+#define CHAR_CR                 '\n'
+
 /* pmk.conf separators */
 #define CHAR_ASSIGN_UPDATE	'='
 #define CHAR_ASSIGN_STATIC	':'
