@@ -1,23 +1,23 @@
-/* $Id: gen_comp_test.c,v 1.1 2004/01/04 20:03:48 mipsator Exp $ */
+/* $Id: gen_comp_test.c,v 1.4 2004/12/26 20:59:45 xsa Exp $ */
 
 /* Public Domain */
 
 /* generate compiler detection test file */
 
-/*#define DEBUG_PRS	1*/
+/*#define PRS_DEBUG	1*/
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "../compat.c"
-#include "../common.c"
-#include "../detect.c"
-#include "../dynarray.c"
-#include "../hash.c"
-#include "../parse.c"
-#include "../pmk_obj.c"
+#include "../common.h"
+#include "../compat/compat.h"
+#include "../detect.h"
+#include "../dynarray.h"
+#include "../hash.h"
+#include "../parse.h"
+#include "../pmk_obj.h"
 
-int main() {
+int main(void) {
 	FILE		*fp;
 	comp_data	*cdata;
 
