@@ -1,14 +1,16 @@
-/* $Id: mkvar_test.c,v 1.2 2003/12/30 21:11:42 mipsator Exp $ */
+/* $Id: mkvar_test.c,v 1.4 2004/12/26 20:59:45 xsa Exp $ */
 
 /* Public Domain */
 
 /* make variables test */
 
-#include "../common.c"
-#include "../compat.c"
-#include "../dynarray.c"
+#include <stdio.h>
 
-int main() {
+#include "../common.h"
+#include "../compat/compat.h"
+#include "../dynarray.h"
+
+int main(void) {
 	char	tmp[256];
 
 	get_make_var("CC", tmp, 256);
