@@ -1,11 +1,15 @@
-/* $Id: path_test.c,v 1.2 2003/12/30 21:11:42 mipsator Exp $ */
+/* $Id: path_test.c,v 1.3 2004/11/26 22:42:40 mipsator Exp $ */
 
 /* Public Domain */
 
 /* path tools test */
 
+#include <sys/param.h>
 #include <stdio.h>
-#include "../pathtools.c"
+
+#include "../compat/compat.h"
+#include "../compat/pmk_stdbool.h"
+#include "../pathtools.h"
 
 char	*chk_paths[4] = {
 		"/usr/./local/bin/../lib//",
