@@ -1,4 +1,4 @@
-/* $Id: pkgconfig_test.c,v 1.6 2004/01/12 20:34:41 mipsator Exp $ */
+/* $Id: pkgconfig_test.c,v 1.8 2004/12/27 11:24:34 xsa Exp $ */
 
 /* Public Domain */
 
@@ -8,19 +8,20 @@
 /*#define HASH_DEBUG	1*/
 
 #include <stdio.h>
+#include <stdlib.h>
 
+#include "../compat/compat.h"
 #include "../compat/pmk_stdbool.h"
-#include "../common.c"
-#include "../dynarray.c"
-#include "../hash.c"
-#include "../parse.c"
-#include "../pkgconfig.c"
-#include "../pmk_obj.c"
+#include "../common.h"
+#include "../dynarray.h"
+#include "../hash.h"
+#include "../parse.h"
+#include "../pkgconfig.h"
+#include "../pmk_obj.h"
 #include "../premake.h"
 
 int main(int argc, char *argv[]) {
-	char	*mod,
-		*pstr;
+	char	*mod;
 	pkgcell	*ppc;
 	pkgdata	*ppd;
 
